{
This object seamlessly integrates three modes of debugging: LCD, RF, and PropPlug.

All that's required to use this object is one of these modes.

User should use the functions provided for CR and CLS
}





VAR       
  Long lcdChars       'The current location of the cursor 
  Byte lcdWidth
  Byte lcdHeight

  Byte  lcd           'The serial port numbers
  Byte  terminal
  Byte  wireless

 

OBJ
  serial : "pcFullDuplexSerial4FCsimple"      

  
PUB start(terminalRx, terminalTx, terminalBaud, wirelessRx, wirelessTx, wirelessBaud, lcdTx, lcdBaud, lcdW, lcdH)

  lcdWidth := lcdW
  lcdHeight := lcdH

  terminal := 0        'Must have ports that can be inputs first in list
  wireless := 1        'The order is the order that the rx's are checked for a byte
  lcd      := 2

  lcdChars := 0
  
  serial.Init
  serial.AddPort(terminal, terminalRx, terminalTx, serial#PINNOTUSED, serial#PINNOTUSED, serial#DEFAULTTHRESHOLD, serial#NOMODE, terminalBaud) 'Starts the terminal link
  serial.AddPort(wireless, wirelessRx, wirelessTx, serial#PINNOTUSED, serial#PINNOTUSED, serial#DEFAULTTHRESHOLD, serial#NOMODE, wirelessBaud) 'Starts the wireless link
  serial.AddPort(lcd     , serial#PINNOTUSED,lcdTx,serial#PINNOTUSED, serial#PINNOTUSED, serial#DEFAULTTHRESHOLD, serial#NOMODE, lcdBaud)      'Starts the LCD    
  serial.start

  serial.tx(lcd, 22) 'Turn off cursor and blink
  serial.tx(lcd, 17) 'Turn on backlight

  waitcnt(clkfreq + cnt)

  cls

  waitcnt(clkfreq + cnt)

PUB stop
  serial.Stop

PUB tx(value)
  {This block takes care of special issues relating to the LCD}
  if(value == 13) 'Carriage Return
    adjustCR
  elseif(value == 200) 'Backspace
    serial.tx(lcd, 8)     'Backup
    serial.tx(lcd, 32)   'Overwrite space
    serial.tx(lcd, 8)     'Backup
    lcdChars--
  else
    adjust(1)
    serial.tx(lcd   , value)
    
  serial.tx(terminal, value)
  serial.tx(wireless, value)

PUB dec(value)   {prints a number in the minimum number of spaces required}

  adjust(decLength(value))
  serial.dec(terminal, value)
  serial.dec(wireless, value)
  serial.dec(lcd     , value)

PUB bin(value, digits)
  adjust(digits)
  serial.bin(terminal, value, digits)
  serial.bin(wireless, value, digits)
  serial.bin(lcd     , value, digits)
  

PUB hex(value, digits)
  adjust(digits)
  serial.hex(terminal, value, digits)
  serial.hex(wireless, value, digits)
  serial.hex(lcd     , value, digits)

PUB str(pointer)

  adjust(strsize(pointer))
 ' waitcnt(clkfreq/2 + cnt)
  serial.str(terminal, pointer)
  'waitcnt(clkfreq/2 + cnt)
  serial.str(wireless, pointer)
 ' waitcnt(clkfreq/2 + cnt)
  serial.str(lcd     , pointer)
 ' waitcnt(clkfreq/2 + cnt)

PUB cr 'cariage return

  adjustCR
  serial.newline(terminal)
  serial.newline(wireless)
  'lcd taken care of in the adjustCR function

PUB cls 'clear screen

  lcdChars := 0
  serial.cls(terminal)
  serial.cls(wireless)
  serial.cls(lcd)  

PUB rx : value | port
{{
Checks the input buffer in the following order:
1. Terminal
2. Wireless

Will wait for input
}}

  port := 1
  value := -1
  repeat while value == -1
    port++
    port //= 2
    value := serial.rxcheck(port)

PUB rxImmediate : value
  {{
  Returns the next byte, terminal priority, wireless second priority.
  Returns -1 if no input
  Does not wait for input.
  }}
  value := serial.rxcheck(terminal)
  if(value <> -1)
    return
  value := serial.rxcheck(wireless)
  

PRI decLength(number) : length | counter
  length := 10
  counter := 1_000_000_000
  if(number == 0)
    length := 0
    return
    
  repeat until (number/counter) <> 0
    counter /= 10   
    length--

  if(number < 0)
    length++

PRI adjust(length) : rollover {adjusts the lcdChars to add in length, and erases the screen if it overflows}
{returns whether or not a rollover (aka, cls on lcd) occurs}

  lcdChars += length
  rollover := false
  if(lcdChars > (lcdWidth*lcdHeight))
    serial.cls(lcd)
    lcdChars := 0
    rollover := true

PRI adjustCR {adjusts the lcdChars to next row}

  ifnot(adjust(lcdWidth - (lcdChars // lcdWidth))) 'If it has not rolled over
    serial.newline(lcd)

DAT
{{

                                                   TERMS OF USE: MIT License                                                                                                              

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation     
files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    
modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software
is furnished to do so, subject to the following conditions:                                                                   
                                                                                                                              
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
                                                                                                                              
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         

}}